'use strict';

exports.__esModule = true;

var _path = require('path');

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackMerge = require('webpack-merge');

var _webpackMerge2 = _interopRequireDefault(_webpackMerge);

var _workboxWebpackPlugin = require('workbox-webpack-plugin');

var _copyWebpackPlugin = require('copy-webpack-plugin');

var _copyWebpackPlugin2 = _interopRequireDefault(_copyWebpackPlugin);

var _webpackModule = require('./webpack-module.config');

var _webpackModule2 = _interopRequireDefault(_webpackModule);

var _webpackNomodule = require('./webpack-nomodule.config');

var _webpackNomodule2 = _interopRequireDefault(_webpackNomodule);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const shared = argv => {
  const isProd = argv.isProd,
        src = argv.src,
        srcDir = argv.srcDir,
        dest = argv.dest,
        nomodule = argv.nomodule,
        pkg = argv.pkg,
        cwd = argv.cwd,
        https = argv.https,
        workers = argv.workers;


  const IS_MODULE_BUILD = !nomodule;
  const ENV = isProd ? 'production' : 'development';
  const OUTPUT_PATH = isProd ? (0, _path.resolve)(dest || 'dist') : (0, _path.resolve)(src || 'src');

  const HOST = process.env.HOST || argv.host;
  const PORT = process.env.PORT || argv.port;
  const FULL_HOST = `http://${HOST}:${PORT}`;
  const ENTRY = isProd ? [(0, _path.resolve)(src || 'src')] : [OUTPUT_PATH].concat([`webpack-dev-server/client?${FULL_HOST}`, 'webpack/hot/dev-server']);

  const processEnv = {
    NODE_ENV: JSON.stringify(ENV),
    appVersion: JSON.stringify(pkg.version)
  };

  const copyStatics = {
    copyWebcomponents: [{
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/webcomponents-loader.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/webcomponents-lite.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/webcomponents-sd-ce.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/webcomponents-hi-sd-ce.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }, {
      from: (0, _path.resolve)('./node_modules/@webcomponents/webcomponentsjs/custom-elements-es5-adapter.js'),
      to: (0, _path.join)(OUTPUT_PATH, 'vendor'),
      flatten: true
    }],
    copyOthers: [{
      from: 'assets/**',
      context: (0, _path.resolve)(srcDir),
      to: OUTPUT_PATH
    }, {
      from: (0, _path.resolve)(srcDir, 'index.html'),
      to: OUTPUT_PATH,
      flatten: true
    }, {
      from: (0, _path.resolve)(srcDir, 'manifest.json'),
      to: OUTPUT_PATH,
      flatten: true
    }]
  };

  const sharedPlugins = [new _webpack2.default.DefinePlugin({ 'process.env': processEnv })];

  const devPlugins = [new _copyWebpackPlugin2.default(copyStatics.copyWebcomponents), new _webpack2.default.HotModuleReplacementPlugin(), new _webpack2.default.NamedModulesPlugin()];

  const buildPlugins = [new _copyWebpackPlugin2.default([].concat(copyStatics.copyWebcomponents, copyStatics.copyOthers))].concat(workers ? new _workboxWebpackPlugin.GenerateSW({
    globDirectory: OUTPUT_PATH,
    globPatterns: ['**/!(*map*)'],
    swDest: (0, _path.join)(OUTPUT_PATH, 'sw.js')
  }) : []);

  const plugins = sharedPlugins.concat(isProd ? buildPlugins : devPlugins);

  return {
    mode: ENV,
    entry: ENTRY,
    output: {
      path: OUTPUT_PATH,
      filename: IS_MODULE_BUILD ? 'module.bundle.js' : 'bundle.js'
    },
    resolve: {
      modules: ['node_modules', (0, _path.resolve)(__dirname, '../../../node_modules')]
    },
    resolveLoader: {
      modules: [(0, _path.resolve)(__dirname, '../../../node_modules'), (0, _path.resolve)(cwd, 'node_modules')]
    },
    module: {
      rules: [{
        test: /\.html$/,
        use: ['raw-loader']
      }, {
        test: /\.pcss$/,
        use: ['raw-loader', 'postcss-loader']
      }]
    },
    plugins,
    devServer: {
      hot: true,
      compress: true,
      contentBase: OUTPUT_PATH,
      overlay: {
        errors: true
      },
      stats: 'minimal',
      port: PORT,
      host: HOST,
      https,
      disableHostCheck: true,
      historyApiFallback: true,
      quiet: true,
      clientLogLevel: 'none',
      watchOptions: {
        ignored: [(0, _path.resolve)(cwd, 'dist'), (0, _path.resolve)(cwd, 'node_modules')]
      }
    }
  };
};

exports.default = (argv = {}) => (0, _webpackMerge2.default)(argv.nomodule ? (0, _webpackNomodule2.default)(argv) : (0, _webpackModule2.default)(argv), shared(argv));